package issrg.globus;

import org.opensaml.SAMLAuthorizationDecisionQuery;
import org.opensaml.SAMLSubject;
import org.opensaml.SAMLAction;
import org.opensaml.SAMLRequest;
import org.opensaml.SAMLResponse;
import org.opensaml.SAMLAuthorizationDecisionStatement;
import org.opensaml.SAMLAssertion;
import org.opensaml.SAMLStatement;

import java.io.ByteArrayInputStream;

/**
* This class represents a Saml AEF that will query a Saml ADF using
* SAML queries.
*/
public class SamlAEF {
	public static final String HELP_SCREEN = "usage: SamlAEF [host [port]]\n\nIf the host is not specified, it will use Localhost. If port is not\nspecified, it will use 80.\n\n";

	private static java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
	private static int port = 80; // default port to connect to

	public static void main(String [] args) throws Exception {
		if (args.length>2 || args.length==0){
			System.out.println(HELP_SCREEN);
			if (args.length>2){
				System.exit(0);
			}
		}

		if (args.length==2){
			if (args[1].intern()=="g"){ // Grid Service must be contacted then
				port=-1;
			}else{
				port = Integer.parseInt(args[1]);
			}
		}

		issrg.globus.impl.PermisAuthz send=port<0?(issrg.globus.impl.PermisAuthz)new GridSamlSender(args[0]):(issrg.globus.impl.PermisAuthz)new SocketSamlSender(args.length==0?"localhost":args[0], port);

		String userDN="";
		String targetDN="";
		String action="";

		while (true){
			String s = readLine("\n\n*** ACTION REQUEST ***\n\nUser DN ["+userDN+"]: ");
			if (s.intern()=="."){
				break;
			}

			if (s.intern()!="") userDN=s;

			s=readLine("Target DN ["+targetDN+"]: ");
			if (s.intern()!="") targetDN=s;

			s=readLine("Action ["+action+"]: ");
			if (s.intern()!="") action=s;

			java.util.Vector actions = new java.util.Vector();
			actions.add(new SAMLAction("", action));

			SAMLAuthorizationDecisionQuery adq = new SAMLAuthorizationDecisionQuery(
						new SAMLSubject(userDN, null, null, null, null, null),
						targetDN,
						actions,
						null
					);

			SAMLRequest r = new SAMLRequest(null, adq, null, null);
			String reqId = r.getId();

			System.out.print("encoding...");
			String str = r.toString();
			System.out.println("done\n"+str+"\n---");

			String rep=send.processSAMLRequest(str);

			System.out.println("decoding..."+rep);

			SAMLResponse resp = new SAMLResponse(new ByteArrayInputStream(rep.getBytes()));
			System.out.println("done\n---");

			System.out.println("checking in-response-to id...");
			//if (resp.getId().intern()!=reqId.intern()){
			//	System.out.println("in-response-to ID is not the same as request ID: "+resp.getId()+" vs "+reqId);
			//}

			System.out.print("retrieving assertions...");
			java.util.Iterator it = resp.getAssertions();
			System.out.print("done\nprint the contents...");

			while(it.hasNext()){
				SAMLAssertion ass = (SAMLAssertion)it.next();
				System.out.println("Validity: "+ass.getNotBefore()+" .. "+ass.getNotOnOrAfter()+
						"\nIssuer: "+ass.getIssuer()+
						"\nStatements: ");

				
				java.util.Iterator i = ass.getStatements();

				while (i.hasNext()){
					SAMLStatement st = (SAMLStatement)i.next();

					System.out.print("checking if it is a AuthorizationDecisionStatement...");
					if (st instanceof SAMLAuthorizationDecisionStatement){
						System.out.print("done\ncasting to SAMLAuthorizationDecisionStatement...");
						SAMLAuthorizationDecisionStatement ads=(SAMLAuthorizationDecisionStatement)st;
						System.out.print("done\noutput the decision details...");

						System.out.println("Decision about action "+
							((SAMLAction)ads.getActions().next()).getData()+": "+
							ads.getDecision());
					}else{
						System.out.print("done\nprinting the raw statement...");
						System.out.println(st.toString());
					}
				}
			}
		}
	}


 	public static String readLine(String prompt){
		System.out.print(prompt);
		String s = null;

		try{
			s=in.readLine();
		}catch(java.io.IOException ioe){
		}

		return s==null?"":s;
	}
}
